//+------------------------------------------------------------------+
//|                                                   iCrosshair.mq4 |
//|                                          Copyright 2015, Awran5. |
//|                                                 awran5@yahoo.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Awran5."
#property link      "awran5@yahoo.com"
#property version   "1.04"
#property strict
#property indicator_chart_window

//-- 1.01 Added: Option to remove tooltip
//-- 1.02 Added: Range
//-- 1.03 Added: Option not to draw horizontal line
//-- 1.04 Shows actual open date for bars on weekly timeframe

input bool              ShowTooltip = true; // Show Tooltip
input string            starttime = "0"; //Start Date
input bool              distancetoHL=true; // Calculate distance to HL?
input string            Header1="";//Chart must contain max 1 horizontal line.
input color             Color  = clrDarkOrange; // Line color
extern ENUM_LINE_STYLE  Style  = 2;         // Line style
extern int              Width  = 1;         // Line width
extern bool             HLine = false; // Draw horizontal line?
extern int              HShift = 10; // Number of bars from current Bar to the right
extern int              VShift = 30; // Number of pips to move H line down
input int               BrokerZone = 3;     // Broker Time Zone
input int               LocalZone = 3;      // Local Time Zone

double pips2double;
string units, date_time;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
   if(Digits%2==1) pips2double=Point*10;
   if(Digits%2==0) pips2double=Point;
   if(Digits==2)   pips2double=1;
   if(Digits==4)   pips2double=Point;

   if (HLine)                    Draw("iCrosshair H Line",OBJ_HLINE,Time[0]+HShift*60*Period(),High[1]-VShift*pips2double,Color,Style,Width,"Click me!");
   if(StringToTime(starttime)>0) Draw("iCrosshair V Line",OBJ_VLINE,iTime(_Symbol,_Period,iBarShift(_Symbol,_Period,StringToTime(starttime))),0,Color,Style,Width,"Click me!");
   if(starttime=="0")            Draw("iCrosshair V Line",OBJ_VLINE,Time[0]+HShift*60*Period(),0,Color,Style,Width,"Click me!");
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| deinitialization function                                        |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   ObjectDelete("iCrosshair H Line");
   ObjectDelete("iCrosshair V Line");
//---
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---

//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
//---
   static bool keyPressed=false;
   if(id==CHARTEVENT_OBJECT_CLICK)
     {
      if(sparam=="iCrosshair H Line" || sparam=="iCrosshair V Line")
        {
         if(!keyPressed) keyPressed=true;
         else keyPressed=false;
        }
      if(keyPressed) ChartSetInteger(0,CHART_EVENT_MOUSE_MOVE,true);
      else ChartSetInteger(0,CHART_EVENT_MOUSE_MOVE,false);
     }
//--- this is an event of a mouse move on the chart
   if(id==CHARTEVENT_MOUSE_MOVE)
     {
      //--- Prepare variables
      int      x     =(int)lparam;
      int      y     =(int)dparam;
      datetime time  =0;
      double   price =0;
      int      window=0;
      //--- Convert the X and Y coordinates in terms of date/time
      if(ChartXYToTimePrice(0,x,y,window,time,price))
        {
         int bar=iBarShift(_Symbol,PERIOD_CURRENT,time);
         
         if(_Period>=PERIOD_D1 || _Period>=PERIOD_MN1)   date_time = TimeToString(Time[bar], TIME_DATE);
         if(PERIOD_W1<=_Period && _Period<PERIOD_MN1)    date_time = TimeToString(Time[bar] + 86400, TIME_DATE);
         if(_Period<PERIOD_D1)                           date_time = TimeToString((Time[bar] + ((LocalZone - BrokerZone) * 60 * 60)));
         
         if (Digits==2 || Digits==0) units = " "+SymbolInfoString(Symbol(),SYMBOL_CURRENCY_MARGIN);
         if (Digits==3 || Digits==5 || Digits==4) units = " pips";
         
         string tooltip="";
         if(ShowTooltip && _Period<PERIOD_D1)
           {
            tooltip="Bar Index: "+(string)bar+
                    "\nDate Time: "+date_time+
                    "\n--------"+
                    "\nBody:  "+DoubleToStr(BarBody(bar),1)+units+
                    "\nRange:  "+DoubleToStr(Range(bar),1)+units+
                    "\nUpper wick:  "+DoubleToStr(upWick(bar),1)+units+
                    "\nLower wick:  "+DoubleToStr(dnWick(bar),1)+units+
                    "\n--------"+
                    "\nDistance: "+DoubleToStr (fabs(Close[0]-Open[bar])/pips2double,2)+units+
                    "\nOpen: "+DoubleToStr(Open[bar],Digits)+                    
                    "\nHigh: "+DoubleToStr(High[bar],Digits)+
                    "\nLow: "+DoubleToStr(Low[bar],Digits)+
                    "\nClose: "+DoubleToStr(Close[bar],Digits)+                    
                    "\n--------"+
                    "\nVolume: "+DoubleToStr(Volume[bar],0);
           }
           if(ShowTooltip && PERIOD_D1<=_Period && _Period<=PERIOD_MN1)
           {
            tooltip="Bar Index: "+(string)bar+
                    "\nDate: "+date_time+
                    "\nOpen: "+DoubleToStr(Open[bar],Digits)+
                    "\nUpper wick:  "+DoubleToStr(upWick(bar),1)+units+
                    "\nLower wick:  "+DoubleToStr(dnWick(bar),1)+units+
                    "\nBody:  "+DoubleToStr(BarBody(bar),1)+units+
                    "\nRange:  "+DoubleToStr(Range(bar),1)+units+
                    "\nDistance: "+DoubleToStr(Distance(bar),2)+units;
           }
         //--- draw lines
         if (HLine) Draw("iCrosshair H Line",OBJ_HLINE,time,price,Color,Style,Width,tooltip);
                    Draw("iCrosshair V Line",OBJ_VLINE,time,price,Color,Style,Width,tooltip);
        }
     }
//---
  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//|                                                                  |

double Distance(int i)
   {
      double dist; string hlname="";
      for(int iObj=ObjectsTotal()-1; iObj >= 0; iObj--)
      {
         string objname = ObjectName(iObj);
         if (StringFind(objname,"Horizontal Line", 0) == 0) hlname = objname;}

      if(distancetoHL && hlname!="") dist = fabs((ObjectGetDouble(0,hlname,OBJPROP_PRICE)-Open[i])/pips2double);
      else dist = fabs(Close[0]-Open[i])/pips2double;
      return(dist);
   }

//+------------------------------------------------------------------+
double upWick(int i)
  {
   double upBody=fmax(Open[i],Close[i]);
   return(fabs(High[i] - upBody)/pips2double);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double BarBody(int i)
  {
   double dnBody=fmin(Open[i],Close[i]),
   upBody=fmax(Open[i],Close[i]);
   return(fabs(upBody - dnBody)/pips2double);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double dnWick(int i)
  {
   double dnBody=fmin(Open[i],Close[i]);
   return(fabs(dnBody - Low[i])/pips2double);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double Range(int i)
  {
   double high=fmax(High[i],Low[i]),
   low=fmin(High[i],Low[i]);
   return(fabs(high - low)/pips2double);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void Draw(string name,int type,datetime time,double price,color clr,int style,int width,string tooltip)
  {
//---
   ObjectDelete(0,name);
   ObjectCreate(0,name,type,0,time,price);
   ObjectSet(name,OBJPROP_COLOR,clr);
   ObjectSet(name,OBJPROP_STYLE,style);
   ObjectSet(name,OBJPROP_WIDTH,width);
   ObjectSet(name,OBJPROP_BACK,true);
   ObjectSetString(0,name,OBJPROP_TOOLTIP,tooltip);
   ChartRedraw(0);
//---
  }
//+------------------------------------------------------------------+
